/*
 * LinearizeAll.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.linearize;

import se.chalmers.cs.gf.abssyn.Tree;
import se.chalmers.cs.gf.linearize.gfvalue.*;
import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;
import static se.chalmers.cs.gf.linearize.LinearizeLogger.*;

import java.util.*;

/**
 *  Linearize a tree with a set of linearizers.
 */
public class LinearizeAll implements Linearizer {

        private Linearizers linearizers;

        public LinearizeAll(Linearizers linearizers) {
                this.linearizers = linearizers;
        }

        // FIXME: a little hacky to pretend that this is all one linearization
        public Value linearize(Tree t) {
                StringBuilder sb = new StringBuilder();
                for (Pair<String,Value> p : linearizeWithAll(linearizers, t)) {
                        sb.append(p.fst);
                        sb.append(": ");
                        sb.append(valueAsString(p.snd));
                        sb.append("\n");
                }

                return Record.ss(sb.toString());
        }

        public String getName() {
                return "*all*";
        }

        public boolean tryThisOne() {
                return false;
        }

        private static String valueAsString(Value value) {
                 Record r = (Record)value;
                 Value v = r.get("s"); // FIXME: is there always an s field?
                 return v.toString(); // FIXME: pick the first form if it is a table
        }

        /**
         *  Linearize a value with all the linearizers.
         */
        public static List<Pair<String,Value>> linearizeWithAll(
                Linearizers linearizers, Tree t) {

                List<Pair<String,Value>> rs = new LinkedList<Pair<String,Value>>();
                for (String lang : linearizers.listLinearizers()) {
                        Linearizer l = linearizers.getLinearizer(lang);
                        if (l.tryThisOne()) {
                                log.fine("Linearizing to " + lang);
                                Value v = l.linearize(t);
                                rs.add(pair(lang,v));
                        }
                }
                return rs;
        }

}
